@@ -1,4 +1,4 @@
-/*  $Id: krb5.c 3172 2007-04-23 22:10:16Z rra $
+/*  $Id: krb5.c 3949 2008-03-22 04:32:54Z rra $
 **
 **  Kerberos interface for WebAuth.
 **
@@ -84,7 +84,57 @@ static krb5_error_code
 #endif
 
 
-/* 
+/*
+**  Replacement for krb5_unparse_name_flags for implementations that don't
+**  have it (MIT and older Heimdal).  Only supports the
+**  KRB5_PRINCIPAL_UNPARSE_NO_REALM flag and always assumes that flag is set.
+*/
+#ifndef HAVE_KRB5_UNPARSE_NAME_FLAGS
+#define KRB5_PRINCIPAL_UNPARSE_NO_REALM 1
+static krb5_error_code
+krb5_unparse_name_flags(krb5_context ctx, krb5_principal princ, int flags,
+                        char **name)
+{
+    krb5_error_code code;
+    krb5_principal copy;
+# ifdef HAVE_KRB5_REALM
+    krb5_realm empty = (char *) "";
+    krb5_realm old_realm;
+# endif
+
+    code = krb5_copy_principal(ctx, princ, &copy);
+    if (code != 0)
+        return code;
+# ifdef HAVE_KRB5_REALM
+    code = krb5_princ_set_realm(ctx, copy, &empty);
+    if (code != 0)
+        return code;
+    code = krb5_unparse_name(ctx, copy, name);
+    if (code != 0)
+        return code;
+    code = krb5_princ_set_realm(ctx, copy, old_realm);
+    if (code != 0)
+        return code;
+    code = krb5_free_principal(copy);
+    if (code != 0)
+        return code;
+# else
+    krb5_free_data_contents(ctx, &copy->realm);
+    krb5_princ_set_realm_length(ctx, copy, 0);
+    krb5_princ_set_realm_data(ctx, copy, NULL);
+    code = krb5_unparse_name(ctx, copy, name);
+    if (code != 0)
+        return code;
+    krb5_free_principal(ctx, copy);
+# endif
+    if ((*name)[strlen(*name) - 1] == '@')
+        (*name)[strlen(*name) - 1] = '\0';
+    return 0;
+}
+#endif
+
+
+/*
 **  Open up a keytab and return a krb5_principal to use with that keytab.  If
 **  in_principal is NULL, returned out_principal is first principal found in
 **  keytab.
@@ -110,7 +160,7 @@ open_keytab(WEBAUTH_KRB5_CTXTP *c, const char *keytab_path,
 
 
     if (in_principal != NULL) {
-        c->code = krb5_parse_name(c->ctx, (char *)in_principal, 
+        c->code = krb5_parse_name(c->ctx, (char *)in_principal,
                                   out_principal);
     } else {
         c->code = krb5_kt_start_seq_get(c->ctx, id, &cursor);
@@ -122,7 +172,7 @@ open_keytab(WEBAUTH_KRB5_CTXTP *c, const char *keytab_path,
 
         c->code = krb5_kt_next_entry(c->ctx, id, &entry, &cursor);
         if (c->code == 0) {
-            c->code = krb5_copy_principal(c->ctx, entry.principal, 
+            c->code = krb5_copy_principal(c->ctx, entry.principal,
                                           out_principal);
             /* use tcode fromt this point so we don't lose value of c->code */
             /* FIXME: when logging is in, log if error if tcode != 0 */
@@ -130,7 +180,7 @@ open_keytab(WEBAUTH_KRB5_CTXTP *c, const char *keytab_path,
             tcode = krb5_free_keytab_entry_contents(c->ctx, &entry);
 #else
             tcode = krb5_kt_free_entry(c->ctx, &entry);
-#endif 
+#endif
         }
         /* FIXME: when logging is in, log if error if tcode != 0 */
         tcode = krb5_kt_end_seq_get(c->ctx, id, &cursor);
@@ -151,8 +201,8 @@ open_keytab(WEBAUTH_KRB5_CTXTP *c, const char *keytab_path,
 **  Like krb5_mk_req, but takes a principal instead of a service/host.
 */
 static krb5_error_code
-mk_req_with_principal(krb5_context context, krb5_auth_context *auth_context, 
-                      krb5_flags ap_req_options, krb5_principal server, 
+mk_req_with_principal(krb5_context context, krb5_auth_context *auth_context,
+                      krb5_flags ap_req_options, krb5_principal server,
                       krb5_data *in_data, krb5_ccache ccache,
                       krb5_data *outbuf)
 {
@@ -170,7 +220,7 @@ mk_req_with_principal(krb5_context context, krb5_auth_context *auth_context,
     retval = krb5_get_credentials(context, 0, ccache, &creds, &credsp);
     if (retval)
 	goto cleanup_creds;
-    retval = krb5_mk_req_extended(context, auth_context, ap_req_options, 
+    retval = krb5_mk_req_extended(context, auth_context, ap_req_options,
 				  in_data, credsp, outbuf);
     krb5_free_creds(context, credsp);
 
@@ -220,7 +270,7 @@ verify_tgt(WEBAUTH_KRB5_CTXTP *c, const char *keytab_path,
     c->code = krb5_rd_req(c->ctx, &auth, &outbuf, server, keytab, NULL, NULL);
     if (auth != NULL)
         krb5_auth_con_free(c->ctx, auth);
-                          
+
     krb5_free_data_contents(c->ctx, &outbuf);
     krb5_kt_close(c->ctx, keytab);
 
@@ -233,6 +283,69 @@ verify_tgt(WEBAUTH_KRB5_CTXTP *c, const char *keytab_path,
 }
 
 
+/*
+**  Given a portion of a Kerberos v5 principal and its length, allocate a new
+**  string and write the escaped form of that portion of a principal to the
+**  newly allocated string.  Returns the newly allocated string, which the
+**  caller is responsible for freeing, or NULL on failure.
+*/
+static char *
+escape_principal(const char *in, size_t length)
+{
+    const char *src;
+    char *out, *dest;
+    size_t needed = 0;
+
+    /* We take two passes through the data, one to count the size of the newly
+       allocated string and one to copy it over. */
+    for (src = in; (src - in) < length; src++) {
+        switch (*src) {
+        case '/':  case '@':  case '\\': case '\t':
+        case '\n': case '\b': case '\0':
+            needed++;
+            break;
+        default:
+            break;
+        }
+        needed++;
+    }
+    out = malloc(needed + 1);
+    if (out == NULL)
+        return NULL;
+    for (src = in, dest = out; (src - in) < length; src++) {
+        switch (*src) {
+        case '/':
+        case '@':
+        case '\\':
+            *dest++ = '\\';
+            *dest++ = *src;
+            break;
+        case '\t':
+            *dest++ = '\\';
+            *dest++ = 't';
+            break;
+        case '\n':
+            *dest++ = '\\';
+            *dest++ = 'n';
+            break;
+        case '\b':
+            *dest++ = '\\';
+            *dest++ = 'b';
+            break;
+        case '\0':
+            *dest++ = '\\';
+            *dest++ = '0';
+            break;
+        default:
+            *dest++ = *src;
+            break;
+        }
+    }
+    *dest = '\0';
+    return out;
+}
+
+
 /*
 **  Create a new WEBAUTH_KRB5_CTXT.
 */
@@ -322,11 +435,11 @@ webauth_krb5_init_via_password(WEBAUTH_KRB5_CTXT *context,
     }
 
     c->code = krb5_cc_resolve(c->ctx, cache_name, &c->cc);
-    if (c->code != 0) 
+    if (c->code != 0)
         return WA_ERR_KRB5;
 
     c->code = krb5_cc_initialize(c->ctx, c->cc, c->princ);
-    if (c->code != 0) 
+    if (c->code != 0)
         return WA_ERR_KRB5;
 
     krb5_get_init_creds_opt_init(&opts);
@@ -394,7 +507,7 @@ webauth_krb5_init_via_cache(WEBAUTH_KRB5_CTXT *context,
     else
         c->code = krb5_cc_default(c->ctx, &c->cc);
 
-    if (c->code != 0) 
+    if (c->code != 0)
         return WA_ERR_KRB5;
 
     c->code = krb5_cc_get_principal(c->ctx, c->cc, &c->princ);
@@ -408,7 +521,7 @@ webauth_krb5_init_via_cache(WEBAUTH_KRB5_CTXT *context,
 **  cache will just be closed, not destroyed, when the context is freed.
 */
 int
-webauth_krb5_keep_cred_cache(WEBAUTH_KRB5_CTXT *context) 
+webauth_krb5_keep_cred_cache(WEBAUTH_KRB5_CTXT *context)
 {
     WEBAUTH_KRB5_CTXTP *c = (WEBAUTH_KRB5_CTXTP *) context;
     assert(c != NULL);
@@ -422,7 +535,7 @@ webauth_krb5_keep_cred_cache(WEBAUTH_KRB5_CTXT *context)
 */
 int
 webauth_krb5_free(WEBAUTH_KRB5_CTXT *context)
-{    
+{
     WEBAUTH_KRB5_CTXTP *c = (WEBAUTH_KRB5_CTXTP *) context;
     assert(c != NULL);
 
@@ -497,7 +610,7 @@ webauth_krb5_init_via_keytab(WEBAUTH_KRB5_CTXT *context,
         krb5_free_principal(c->ctx, c->princ);
 
     s = open_keytab(c, keytab_path, server_principal, &c->princ, &keytab);
-    if (s != WA_ERR_NONE) 
+    if (s != WA_ERR_NONE)
         return WA_ERR_KRB5;
 
     if (cache_name == NULL) {
@@ -570,7 +683,7 @@ webauth_krb5_init_via_cred(WEBAUTH_KRB5_CTXT *context, char *cred,
 
     s = cred_from_attr_encoding(c, cred, cred_len, &creds);
 
-    if (s != WA_ERR_NONE) 
+    if (s != WA_ERR_NONE)
         return s;
 
     if (cache_name == NULL) {
@@ -614,7 +727,7 @@ webauth_krb5_import_cred(WEBAUTH_KRB5_CTXT *context, char *cred, int cred_len)
     assert(cred != NULL);
 
     s = cred_from_attr_encoding(c, cred, cred_len, &creds);
-    if (s!= WA_ERR_NONE) 
+    if (s!= WA_ERR_NONE)
         return s;
 
     /* Add the creds to the cache. */
@@ -686,33 +799,86 @@ webauth_krb5_service_principal(WEBAUTH_KRB5_CTXT *context, const char *service,
 
 
 /*
-**  Get the principal from a context.  If the local flag is set to true, run
-**  the principal through krb5_aname_to_localname first to try to generate a
-**  local username.  Fall through a fully-qualified name.
+**  Get the principal from a context.
+**
+**  Principal canonicalization is controlled by the third argument.  If it's
+**  WA_KRB5_CANON_NONE, do no canonicalization.  If it's WA_KRB5_CANON_LOCAL,
+**  run the principal through krb5_aname_to_localname first to try to generate
+**  a local username and fall through a fully-qualified name.  If it's
+**  WA_KRB5_CANON_STRIP, strip the realm from the principal, whatever it may
+**  be.
 */
 int
 webauth_krb5_get_principal(WEBAUTH_KRB5_CTXT *context, char **principal,
-                           int local)
+                           int canonicalize)
 {
     WEBAUTH_KRB5_CTXTP *c = (WEBAUTH_KRB5_CTXTP *) context;
+    krb5_error_code tcode;
+    char lname[256];
 
     if (c->princ == NULL)
         return WA_ERR_INVALID_CONTEXT;
 
-    if (local) {
-        krb5_error_code tcode;
-        char lname[256];
-
+    switch (canonicalize) {
+    case WA_KRB5_CANON_LOCAL:
         tcode = krb5_aname_to_localname(c->ctx, c->princ, sizeof(lname) - 1,
                                         lname);
         if (tcode == 0) {
             *principal = malloc(strlen(lname) + 1);
             strcpy(*principal, lname);
             return WA_ERR_NONE;
-        } 
+        }
+        /* Fall through. */
+    case WA_KRB5_CANON_NONE:
+        c->code = krb5_unparse_name(c->ctx, c->princ, principal);
+        break;
+    case WA_KRB5_CANON_STRIP:
+        c->code = krb5_unparse_name_flags(c->ctx, c->princ,
+                                          KRB5_PRINCIPAL_UNPARSE_NO_REALM,
+                                          principal);
+        break;
     }
-
-    /* Fall through to fully-qualified on krb5_aname_to_localname errors. */
-    c->code = krb5_unparse_name(c->ctx, c->princ, principal);
     return c->code == 0 ? WA_ERR_NONE : WA_ERR_KRB5;
 }
+
+
+/*
+**  Get the authentication realm from the context.  Stores the newly allocated
+**  string in realm and returns WA_ERR_NONE on success, or another error code
+**  on failure.
+*/
+int
+webauth_krb5_get_realm(WEBAUTH_KRB5_CTXT *context, char **realm)
+{
+    WEBAUTH_KRB5_CTXTP *c = (WEBAUTH_KRB5_CTXTP *) context;
+    char *raw;
+    size_t length;
+#ifdef HAVE_KRB5_REALM
+    krb5_realm *data;
+#else
+    krb5_data *data;
+#endif
+
+    if (c->princ == NULL)
+        return WA_ERR_INVALID_CONTEXT;
+
+    /* Extract the realm from the principal.  This works differently in MIT
+       and Heimdal. */
+#ifdef HAVE_KRB5_REALM
+    data = krb5_princ_realm(c->ctx, c->princ);
+    if (data == NULL)
+        return WA_ERR_INVALID_CONTEXT;
+    raw = krb5_realm_data(data);
+    length = krb5_realm_length(data);
+#else
+    data = krb5_princ_realm(c->ctx, c->princ);
+    if (data == NULL)
+        return WA_ERR_INVALID_CONTEXT;
+    raw = data->data;
+    length = data->length;
+#endif
+
+    /* Encode and return the realm data. */
+    *realm = escape_principal(raw, length);
+    return (*realm == NULL) ? WA_ERR_NO_MEM : WA_ERR_NONE;
+}
