@@ -32,7 +32,7 @@ int main(int argc, char *argv[])
     char *tgt, *ticket;
     int tgtlen, ticketlen;
     time_t expiration;
-    char *cprinc;
+    char *cprinc, *crealm;
 
     if (argc != 6) {
         usage();
@@ -45,8 +45,9 @@ int main(int argc, char *argv[])
     host = argv[5];
 
     cprinc = NULL;
+    crealm = NULL;
 
-    START_TESTS(25);
+    START_TESTS(32);
 
     s = webauth_krb5_new(&c);
     TEST_OK2(WA_ERR_NONE, s);
@@ -55,6 +56,8 @@ int main(int argc, char *argv[])
     /* test failure case */
     s = webauth_krb5_get_principal(c, &cprinc, 1);
     TEST_OK2(WA_ERR_INVALID_CONTEXT, s);
+    s = webauth_krb5_get_realm(c, &crealm);
+    TEST_OK2(WA_ERR_INVALID_CONTEXT, s);
 
     s = webauth_krb5_init_via_password(c, username, password, 
                                        keytab_path, NULL,
@@ -63,9 +66,22 @@ int main(int argc, char *argv[])
     TEST_OK2(WA_ERR_NONE, s);
     TEST_OK(server_principal != NULL);
 
-    s = webauth_krb5_get_principal(c, &cprinc, 1);
+    s = webauth_krb5_get_principal(c, &cprinc, WA_KRB5_CANON_LOCAL);
     /*printf("cprinc = %s\n", cprinc);*/
     TEST_OK2(WA_ERR_NONE, s);
+    TEST_OK(strchr(cprinc, '@') == NULL);
+    free(cprinc);
+    s = webauth_krb5_get_principal(c, &cprinc, WA_KRB5_CANON_NONE);
+    TEST_OK2(WA_ERR_NONE, s);
+    TEST_OK(strchr(cprinc, '@') != NULL);
+    free(cprinc);
+    s = webauth_krb5_get_principal(c, &cprinc, WA_KRB5_CANON_STRIP);
+    TEST_OK2(WA_ERR_NONE, s);
+    TEST_OK(strchr(cprinc, '@') == NULL);
+
+    s = webauth_krb5_get_realm(c, &crealm);
+    /*printf("crealm = %s\n", crealm);*/
+    TEST_OK2(WA_ERR_NONE, s);
 
     /*
     printf("code(%d) mess(%s)\n", 
